#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include "Types.h"

int DEBUG = 0;

int gNcores           = 4; // 4 CPUs is default

int gPDBxyz           = 0;

int gFrom             = 0;
int gTo               = -1;
int gToSetup          = 0;
int gFLen             = 0;
int gLenSetup         = 0;

double gGlobalScore   = 0.6;
double gPValue        = 0.05;
double gBCScore       = -1.0;
double gBCDist        = 0.3;

DtStr IFragFichName    = "";
DtStr IProtFichName    = "";
DtStr OLogFichName    = "stderr";
DtStr OFichName       = "stdout";

int gVerbose = 0;

void delimiter(FILE *f)
{
  fprintf(f,"----------------------------------------------------\n");
}

/* ------------------------------------------------------------
 * Possibles arguments de la ligne de commande. --------------- 
 * ------------------------------------------------------------
 */
void printUsage()
{
  fprintf(stderr,"BCSearch: a program to score the similarity between two protein fragments (of identical size).\n");
  fprintf(stderr,"       F. Guyon and P. Tuffery, 2011-2013\n\n");
  fprintf(stderr," Data input is in the form of xy files (3 coordinates per line)\n");
  fprintf(stderr," The -pdbxyz option corresponds to strict PDB format, allowing coordinate numbers without blanks as separators\n\n");
  fprintf(stderr,"  usage: BCSearch <args>\n");
  fprintf(stderr,"\nGeneric parameters:\n");
  fprintf(stderr,"         -if   <xyzFich>    : input query fragment to search (xyz format - three numbers blank separated) (stdin)\n");
  fprintf(stderr,"         -ip   <xyzFich>    : input protein to scan for similar fragments (xyz format - three numbers blank separated) (stdin)\n");
  fprintf(stderr,"         -pdbxyz            : switch for reading using the xyz 8.3 format (as in PDB files) (stdin)\n");
  
  fprintf(stderr,"\nmisc. parameters:\n");
  fprintf(stderr,"         -h                 : To get this information\n");
  //  fprintf(stderr,"         -v                 : verbose\n");
}

void parseArgstr(int argc,char *argv[])
{
  int i;

  if (argc == 1) {
    printUsage();
    exit(0);
  }

  for (i=0;i<argc;i++) {
    
    if (!strcmp(argv[i],"-h")) { /* print usage */
      printUsage();
      exit(0);
    }
    
    /* Le fichier profile seed (input) --------------------- */
    if (!strcmp(argv[i],"-if")) {
      strcpy(IFragFichName,argv[i+1]);
      i++;
    } 
    if (!strcmp(argv[i],"-pdbxyz")) {
      gPDBxyz = 1;
    } 
    /* Le fichier profile seed (input) --------------------- */
    if (!strcmp(argv[i],"-ip")) {
      strcpy(IProtFichName,argv[i+1]);
      i++;
    } 
    /* Le fichier results (output) --------------------- */
    if (!strcmp(argv[i],"-o")) {
      strcpy(OFichName,argv[i+1]);
      i++;
    } 
    /* Le fichier log (output) --------------------- */
    if (!strcmp(argv[i],"-olog")) {
      strcpy(OLogFichName,argv[i+1]);
      i++;
    } 
    /* mode DEBUG --------------------------------------- */
    if (!strcmp(argv[i],"-dbg")) {
      DEBUG = 1;
    } 
    /* mode DEBUG --------------------------------------- */
    if (!strcmp(argv[i],"-v")) {
      gVerbose = 1;
    } 

  }

}


/* =====================================================================
 * Global parameters information
 * =====================================================================
 */
void prmtrsSumary(FILE *f)
{
  delimiter(f);
  fprintf(f,"GLSearch (F. Guyon, P. Tuffery, 2011-2013)\n");
  delimiter(f);
  fprintf(f,"\nSearch parameters\n");
  fprintf(f,"  input fragment file   : %s\n", IFragFichName);
  fprintf(f,"  input protein file    : %s\n", IProtFichName);
  delimiter(f);

  fflush(f);
}

