#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "util.h"
/*
 *
 */
void skipLine (FILE *file) {
char c;
  c=fgetc(file);
  while (c!=EOF && c!='\n') {
     c=fgetc(file);
  }
}

char *readVarLine(FILE *file) {
char *s;
int  n, size;
char c;
  s = (char *)malloc(sizeof(char)*INIT_SIZE);
  n=0;
  size = INIT_SIZE;
  c=fgetc(file);
  while (c!=EOF && c!='\n') {
    s[n]=c;n++;
    if (n>=size) {
      size+=INC_SIZE;
      s=(char *) realloc(s, sizeof(char)*size);
    }
    c=fgetc(file);
  }
  s[n]='\0';
  return s;
}

char **readVarLines(FILE *file, int *len, char *id) {
  char **ss=NULL;
  char *s;
  int n, size;

  ss=(char **) realloc(ss, sizeof(char *)*INIT_SIZE);
  size = INIT_SIZE;
  if (id!=NULL) {readCommentLine(file, &id);}
  s=readVarLine(file); 
  n=0;
  while (strlen(s)>0) {
    ss[n]=s;n++;
    s=readVarLine(file); 
    if (n>=size) {
      size+=INC_SIZE;
      ss=(char **) realloc(ss, sizeof(char *)*size);
    }
  }
  *len=n;
  fclose(file);
  return ss;
}


void freeVarLines(char **ss, int n) {
  int i;

  for (i=0; i<n; i++) 
    free(ss[i]);
  free(ss);
}



int readLine(FILE *file, char *buffer) {
  
int n;
char c;
  n=0;
  c=fgetc(file);
  while (c!=EOF && c!='\n' && c!='>' && n<MAXLINE) {
    buffer[n++]=c;
    c=fgetc(file);
  }
  buffer[n]='\0';
  if (c==EOF || c=='\n') return n;
  if (n>=MAXLINE) return -MAXLINE;
  if (c=='>') { fseek(file, -1, SEEK_CUR);return n;}
}

int readCommentLine (FILE *file, char **id) {
char c;
int n=0;

  c=fgetc(file);
  if (c=='>') {
    c=fgetc(file);
    while (c!=EOF && c!='\n' && n<MAXLINE) {
      (*id)[n++]=c;
      c=fgetc(file);
    }
    (*id)[n]='\0';
  }
  return n;
}

#if 0
int parseDoubleVector(char *buffer, double *x) {
  char *pstr, *str;
  int n;

  pstr=buffer;
  n=0;
  while ((str=strsep(&pstr, " \t"))!=NULL && n<MAXROW){
    if (strlen(str)>0) x[n++]=atof(str);
  }
  return n;
}

#else

int parseDoubleVector(char *buffer, double *x, int isPDBxyz) {
  char *pstr, *str;
  int n;
  if (isPDBxyz) {
    return parseDoubleVectorPDBxyz(buffer,x);
  }
  pstr=buffer;
  n=0;
  while ((str=strsep(&pstr, " "))!=NULL && n<MAXROW){
    if (strlen(str)>0) x[n++]=atof(str);
  }
  return n;
}

int parseDoubleVectorPDBxyz(char *buffer, double *x) {
  char charX[9],charY[9],charZ[9];

  if (sscanf(buffer,"%8c%8c%8c",charX,charY,charZ) != EOF);
  x[0] = atof(charX);
  x[1] = atof(charY);
  x[2] = atof(charZ);
  return 3;
}

#endif

DoubleVector *newDoubleVector(int n) {
  DoubleVector *X;
  double *x;
  X=(DoubleVector *)calloc(1, sizeof(DoubleVector));
  x=(double *)calloc(n, sizeof(double));
  X->values=x;
  X->size=n;
  return X;
}

void freeDoubleVector(DoubleVector *X) {
  free(X->values);
  free(X);
}

DoubleVector *reallocDoubleVector(DoubleVector *X, int n) {
  X->values = (double *) realloc(X->values, n*sizeof(double));
  X->size=n;
  return X;
}

#if 0
/*
 * si X!= NULL : append to X new read values
 * si id!=NULL : read first line as a comment line
 */
DoubleVector *readDoubleVector(FILE *file, DoubleVector *X, int *status, char *id) {
  char buffer[MAXLINE];
  double x[MAXROW];
  int i, j, n,  size;
  int c;
  if (X==NULL) {
    X=newDoubleVector(INIT_SIZE);
  }
  if (id!=NULL) readCommentLine(file, &id);
  size=0;
  c=readLine(file, buffer);
  while (c>0) {
    n=parseDoubleVector(buffer, x);
    if (size+n>X->size) {
      X=reallocDoubleVector(X, X->size+INC_SIZE);
    }
    for (i=0; i<n; i++) { 
      X->values[size+i]=x[i];
    }
    size+=n;
    c=readLine(file, buffer);
  }
  X->size=size;
  *status=X->size;
  return X;
}
#else
/*
 * si X!= NULL : append to X new read values
 * si id!=NULL : read first line as a comment line
 * la premiere ligne donne le nombre de colonne
 */
DoubleVector *readDoubleVector(FILE *file, DoubleVector *X, int *ncol, char *id, int isPDBxyz) {
  char buffer[MAXLINE];
  double x[MAXROW];
  int i, j, n,  n0, size;
  int c;
  if (X==NULL) {
    X=newDoubleVector(INIT_SIZE);
  }
  if (id!=NULL) readCommentLine(file, &id);
  size=0;
  c=readLine(file, buffer);
  if (c>0) n0=parseDoubleVector(buffer, x, isPDBxyz); 
  n=n0;
  while (c>0 && n==n0) {
    if (size+n>X->size) {
      X=reallocDoubleVector(X, X->size+INC_SIZE);
    }
    for (i=0; i<n; i++) { 
      X->values[size+i]=x[i];
    }
    size+=n;
    c=readLine(file, buffer);
    if (c>0) n=parseDoubleVector(buffer, x, isPDBxyz);
  }
  X->size=size;
  *ncol=n0;
  return X;
}
#endif
  
void printDoubleVector(DoubleVector *X, int ncol) {
  int i;

  for (i=0; i<X->size; i++) {
    if (i%ncol==0) printf("\n");
    printf("%lf ", X->values[i]);
  }
  printf("\n");
}
